/* coincoinpost.cs : this file is part of coincoin.net
 * (c) 2002, kadreg
 */
namespace org.kadreg.coincoin {
  using System.Drawing;
  using  System.Collections;

  /// <summary>Les types de trucs interressants que
  /// l'on peut trouver dans le pinnipede.</summary>
  public enum InterrestingThingType {
    Nothing,
    LeftHorloge,
    Horloge, 
    URL,
    Login,
    Word
  }

  /// <summary> Un truc interressant dans le pinnipede.
  /// avec son type et sa valeur
  ///</summary>
  public struct InterrestingThing {
    public InterrestingThingType type;
    public string value;
  }

  public class CoinCoinUrl {
    int indexDebut;
    int indexFin;
    string url;
    
    public CoinCoinUrl (int i, string s) {
      indexFin = i;
      indexDebut = i;
      url = s;
    }

    public void setIndexFin (int i) {
      indexFin = i;
    }

    public string getUrl () {
      return url;
    }
    
    public bool isInUrl (int index) {
      return (index >= indexDebut) && (index <= indexFin);
    }
  }

  public class CoinCoinPost {
    /* attributs internes */
    int _Id;    
    string _Login;
    string _Info;
    string _Message;
    string _Timestamp;
    string _unformatedMessage;
    ArrayList _UrlList; // les URL du post

    /* taille d'horloge */
    const int tailleHorloge = 8;

    /* index de dbut d'affichage */
    int _startIndex = -1;

    /* une URL en cours de construction. Declaree
     par createUrl, mais pas encore valide par
    commitUrl */
    CoinCoinUrl pendingUrl = null;

    /* proprietes pour acceder aux attributs */
    public int Id {
      get {
	return _Id;
      }
      set{
	_Id = value;
      }
    }
    public string Login {
      get {
	return _Login;
      }
      set{
	_Login = value;
      }
    }
    public string Info {
      get {
	return _Info;
      }
      set{
	_Info = value;
      }
    }
    public string Message {
      get {	
	return _Message;
      }
      set{
	_Message = value;
      }
    }
    public string Timestamp {
      get {
	return formatTimestamp(_Timestamp);
      }
      set{
	_Timestamp = value;
      }
    }    
    public int Index {
      get {
	return _startIndex;
      }
      set {
	_startIndex = value;
      }
    }

    public string UnformatedMessage {
      get {
	return _unformatedMessage;
      }
    }

    public CoinCoinPost (int i, 
			 string l, 
			 string inf, 
			 string m, 
			 string t) {
      Id = i;
      Login = l;
      Info = inf;
      Message = m;
      Timestamp = t;
      _UrlList = null;
    }

    /// <summary>
    /// ajoute du texte non formate a la version non html de Message
    ///</summary>
    public void appendUnformatedText (string str) {
      _unformatedMessage += str;
    }

    public string formatTimestamp (string ts) {
      string str = ts.Substring (8, 2) + ":"
	+ ts.Substring (10, 2) + ":"
	+ ts.Substring (12, 2);
      return str;
    }

    public void writeInPinnipede (PinnipedeBox box) {
      string text;
      Index = box.TextLength;
      box.SelectionColor = Color.DarkBlue; 
      box.AppendText (Timestamp); 
      box.AppendText (" ");
      if (Login != "Anonyme") {
	box.SelectionColor = Color.Red; 
	box.AppendText (System.Web.HttpUtility.HtmlDecode (Login)); 
      } else {
	box.SelectionColor = Color.DarkRed; 
	text = Info;
	text = System.Web.HttpUtility.HtmlDecode (text);
	text = text.Substring (0, (text.Length>=10)?10:text.Length);
	box.AppendText (text);
      }
      box.SelectionColor = Color.Black; 
      box.AppendText (" : "); 
      box.AppendHtmlText (Message, this); 
      box.AppendText ("\n");
    }

    /* 8 caracteres d'horloge, au max 10 d'Info, un ":" et trois espaces */
    public int getShiftBeforeMessage () {
      int res;
      if (Login != "Anonyme") {
	res = 3 + 1 + 8 + Login.Length; 
      } else {
	res = 3 + 1 + (8 + ((Info.Length>=10)?10:Info.Length)); 
      }
      return res;
    }

    public InterrestingThing findInterrestingThingInPost (int index) {
      /* l'utilisateur a cliqu sur l'index index du message. 
	 a quoi cela correspond t'-il ? Une horloge ? Un 
	 Login ? Une url ? 
      */
      string temp;
      InterrestingThing thing;

      /* es-ce une horloge, un login, ? */
      if (index <= tailleHorloge) {
	/* CAI UNE NORLOGE */
	thing.type = InterrestingThingType.LeftHorloge;
	thing.value = Timestamp;
	return thing;
      }

      if ((index > tailleHorloge) && (index < getShiftBeforeMessage ())) {
	/* CAI UN UA */
	thing.type = InterrestingThingType.Login;
	thing.value = this.Login;
	return thing;
      }

      /* c'est peut tre une horloge qui traine dans un message */
      temp = getHorlogeInMessageAtPosition (index - getShiftBeforeMessage ());
      if (temp != "") {
	thing.type = InterrestingThingType.Horloge;
	thing.value = temp;	
	return thing;
      }

      /* c'est peut tre une [URL] qui traine dans un message */
      foreach (CoinCoinUrl u in _UrlList) {
	if (u.isInUrl (index)) {
	  // System.Console.WriteLine ("url found " + u.getUrl ());
	  thing.type = InterrestingThingType.URL;
	  thing.value = u.getUrl ();
	  return thing;
	}
      }
      
      /* CAI UN MOT ? */

      thing.type = InterrestingThingType.Nothing;
      thing.value = "";
      return thing;
    }

    /* lance la creation d'une URL situe a l'index i du post*/
    public void createUrl (int i, string url) {
      // System.Console.WriteLine ("createUrl  " + url + " @index:" + i);
      pendingUrl = new CoinCoinUrl (getShiftBeforeMessage () + i, url);
    }

    /* commit l'url avec l'index de fin i */
    public void commitUrl (int i) {
      //System.Console.WriteLine (" commintUrl@index:" + i);
      if (_UrlList == null) {
	_UrlList = new ArrayList ();
      }
      pendingUrl.setIndexFin (getShiftBeforeMessage () + i);
      _UrlList.Add (pendingUrl);
      pendingUrl = null;
    }

    protected string getHorlogeInMessageAtPosition (int index) {
      string pattern = "[0-2]?\\d:[0-5]\\d(:[0-5]\\d)?";

/*       System.Console.WriteLine ("getHorlogeInMessageAtPosition : index={0} message={1}", */
/* 				index, Message); */
      foreach (System.Text.RegularExpressions.Match match in 
	       System.Text.RegularExpressions.Regex.Matches (Message, pattern)) {
/* 	System.Console.WriteLine ("Index = {2} : Found a match at {0} : {1}",  */
/* 				  match.Index, match.Value, index); */
	
	if ((index > match.Index) 
	    && (index < (match.Index + match.Value.Length))) {
	  /* c'est bien(c), on garde */
	  return match.Value;
	}
      }
      return "";
    }

  }
    
  public class PinnipedeData : System.Collections.SortedList {
    public void addPost (int i, 
			 string l, 
			 string inf, 
			 string m, 
			 string t) {
      CoinCoinPost post = new CoinCoinPost (i, l, inf, m, t);
      Add (i, post);
    }

  }


}
